! (C) Copyright 2009-2016 ECMWF.
! 
! This software is licensed under the terms of the Apache Licence Version 2.0
! which can be obtained at http://www.apache.org/licenses/LICENSE-2.0. 
! In applying this licence, ECMWF does not waive the privileges and immunities 
! granted to it by virtue of its status as an intergovernmental organisation nor
! does it submit to any jurisdiction.

!> Interfaces to be called from C++ for Fortran handling of QG model geometry

! ------------------------------------------------------------------------------

subroutine c_qg_geo_setup(c_key_self, c_conf) bind(c,name='qg_geo_setup_f90')
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int), intent(inout) :: c_key_self
type(c_ptr), intent(in)    :: c_conf

type(qg_geom)          :: template
type(qg_geom), pointer :: self

! -- get objects

call geom_list%add(c_key_self,template)
self => get_geom(c_key_self)

call setup(self, c_conf)

end subroutine c_qg_geo_setup

! ------------------------------------------------------------------------------

subroutine c_qg_geo_set_atlas_lonlat(c_key_self, c_fset) bind(c,name='qg_geo_set_atlas_lonlat_f90')
use atlas_module
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int), intent(inout) :: c_key_self
type(c_ptr), intent(in), value :: c_fset

type(qg_geom), pointer :: self
type(atlas_fieldset) :: fset

! -- get objects

self => get_geom(c_key_self)
fset = atlas_fieldset(c_fset)

call set_atlas_lonlat(self, fset)

end subroutine c_qg_geo_set_atlas_lonlat

! ------------------------------------------------------------------------------

subroutine c_qg_geo_fill_fields(c_key_self, c_fspace, c_fset) &
 & bind(c,name='qg_geo_fill_fields_f90')
use atlas_module
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int),intent(in) :: c_key_self
type(c_ptr),intent(in),value :: c_fspace
type(c_ptr),intent(in),value :: c_fset

type(qg_geom), pointer :: self
type(atlas_functionspace_pointcloud) :: fspace
type(atlas_fieldset) :: fset

! -- get objects

self => get_geom(c_key_self)
fspace = atlas_functionspace_pointcloud(c_fspace)
fset = atlas_fieldset(c_fset)

call fill_fields(self, fspace, fset)

end subroutine c_qg_geo_fill_fields

! ------------------------------------------------------------------------------

subroutine c_qg_geo_clone(c_key_self, c_key_other) bind(c,name='qg_geo_clone_f90')
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int), intent(in   ) :: c_key_self
integer(c_int), intent(inout) :: c_key_other

type(qg_geom)          :: template 
type(qg_geom), pointer :: self, other

! -- get objects

call geom_list%add(c_key_other,template)
other => get_geom(c_key_other)
self => get_geom(c_key_self)

call clone(self, other)

end subroutine c_qg_geo_clone

! ------------------------------------------------------------------------------

subroutine c_qg_geo_delete(c_key_self) bind(c,name='qg_geo_delete_f90')
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int), intent(inout) :: c_key_self     

call geom_list%remove(c_key_self)

end subroutine c_qg_geo_delete

! ------------------------------------------------------------------------------

subroutine c_qg_geo_info(c_key_self, c_nx, c_ny) bind(c,name='qg_geo_info_f90')
use iso_c_binding
use qg_geom_mod
implicit none
integer(c_int), intent(in   ) :: c_key_self
integer(c_int), intent(inout) :: c_nx
integer(c_int), intent(inout) :: c_ny
type(qg_geom), pointer :: self

! -- get objects

self => get_geom(c_key_self)

call grid_info(self, c_nx, c_ny)

end subroutine c_qg_geo_info
